unit ODBF_FDE;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Dialogs, ODBFTool;

type
  TFieldDefEditDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Bevel1: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    NameEdit: TEdit;
    TypeComboBox: TComboBox;
    LengthEdit: TEdit;
    DecimalsEdit: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    DBFFieldDef: TDBFFieldDef;
    procedure ImportDBFFieldDef (NewDef: TDBFFieldDef);
  end;

var
  FieldDefEditDlg: TFieldDefEditDlg;

implementation

uses ODBF;

var IsCloseForm: Boolean;

{$R *.DFM}

procedure TFieldDefEditDlg.FormCreate(Sender: TObject);
var I: Integer;
begin
  TypeComboBox.Items.Clear;
  for I:=0 to MaxType-1 do
    TypeComboBox.Items.Add(TypeStrings[I]);
  DBFFieldDef:=NIL;
end;

procedure TFieldDefEditDlg.ImportDBFFieldDef (NewDef: TDBFFieldDef);
begin
  if DBFFieldDef=NIL then
     begin
       if NewDef<>NIL then
          begin
            if (NewDef is TDBFFieldDef)
               then DBFFieldDef:=NewDef.Copy
               else DBFFieldDef:=TDBFFieldDef.Create('','C',10,0);
          end
       else DBFFieldDef:=TDBFFieldDef.Create('','C',10,0);
     end;
end;

procedure TFieldDefEditDlg.FormActivate(Sender: TObject);
var I: Integer;
    NumStr: String[10];
begin
  if DBFFieldDef=NIL then DBFFieldDef:=TDBFFieldDef.Create('','C',10,0);
  ActiveControl:=NameEdit;
  IsCloseForm:=TRUE;
  with DBFFieldDef do
    begin
      NameEdit.Text:=FieldName;
      TypeComboBox.Text:=FieldType;
      for I:=0 to MaxType-1 do
        if FieldType=TypeChars[I] then
           begin TypeComboBox.ItemIndex:=I; break; end;
      Str(FieldLength,NumStr);
      LengthEdit.Text:=NumStr;
      Str(FieldDecimals,NumStr);
      DecimalsEdit.Text:=NumStr;
    end;
end;

procedure TFieldDefEditDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if IsCloseForm then
     begin
       Action:=caFree;
     end
  else
     begin
       Action:=caNone;
       ActiveControl:=NameEdit;
       IsCloseForm:=True;
     end;
end;

procedure TFieldDefEditDlg.FormDestroy(Sender: TObject);
begin
  DBFFieldDef.Free;
end;

procedure TFieldDefEditDlg.OKBtnClick(Sender: TObject);
var I, Index, Code: Integer;
    FName, S: String;
    FType: Char;
    FLength, FDecimals, MaxLength: Byte;
    Correct: Boolean;
begin
  IsCloseForm:=TRUE;
  {Name check}
  FName:=NameEdit.Text;
  if Length(FName)>0 then
     while FName[Length(FName)]=#$20 do System.Delete(FName,Length(FName),1);
  if Length(FName)>0 then
     while FName[1]=#$20 do System.Delete(FName,1,1);
  if Length(FName)>10 then FName:=Copy(FName,1,10);
  if Length(FName)=0 then
     begin
       MessageDlg('Incorrect field name',mtError,[mbOK],0);
       IsCloseForm:=FALSE;
       exit;
     end;
  {Type check}
  S:=TypeComboBox.Text;
  Index:=-1;
  for I:=0 to TypeComboBox.Items.Count-1 do
    if S=TypeComboBox.Items[I] then
       begin Index:=I; break; end;
  if Index<0 then
     begin
       MessageDlg('Incorrect field type',mtError,[mbOK],0);
       IsCloseForm:=FALSE;
       exit;
     end;
  if Index<MaxType
     then FType:=TypeChars[Index]
     else FType:=TypeChars[0];
  {Length check}
  FLength:=0;
  case FType of
    'D': FLength:=8;
    'L': FLength:=1;
    'M': FLength:=10;
    'C','N':
       begin
         Correct:=TRUE;
         S:=LengthEdit.Text;
         Val(S,Index,Code);
         if Code<>0 then Correct:=FALSE;
         if FType='C'
            then MaxLength:=254
            else MaxLength:=19;
         if Correct then
            if (Index<1) OR (Index>MaxLength) then Correct:=FALSE;
         if Not Correct then
            begin
              MessageDlg('Incorrect field length',mtError,[mbOK],0);
              IsCloseForm:=FALSE;
              exit;
            end
         else FLength:=Index;
       end;
  end;
  {Decimals check}
  if FType='N' then
     begin
       Correct:=TRUE;
       S:=DecimalsEdit.Text;
       Val(S,Index,Code);
       if Code<>0 then Correct:=FALSE;
       if Correct then
          if (Index<0) OR (Index>=FLength) then Correct:=FALSE;
       if Not Correct then
          begin
            MessageDlg('Incorrect field decimals',mtError,[mbOK],0);
            IsCloseForm:=FALSE;
            exit;
          end
       else FDecimals:=Index;
     end
  else FDecimals:=0;
  {DBFFieldDef export}
  with DBFFieldDef do
    begin
      FieldName:=FName;
      FieldType:=FType;
      FieldLength:=FLength;
      FieldDecimals:=FDecimals;
    end;
end;

end.

